unit coll;
{------------------------------------------------------------------------------}
{ Unit:     coll                                                               }
{ Project:  Collection Demo                                                    }
{ Client:   N/A                                                                }
{ Author:   Dan Brown                                                          }
{ Date:     10/03/2001                                                         }
{                                                                              }
{ Description:                                                                 }
{                                                                              }
{ TStringListHolder component written to demonstrate implementing collections  }
{ for UK Borland User Group.                                                   }
{                                                                              }
{ Revision History:                                                            }
{                                                                              }
{ Version   Date        Name          Description                              }
{ --------- ----------  ------------- ---------------------------------------- }
{ 1.00      10/03/2001  Dan Brown     Version 1                                }
{                                                                              }
{------------------------------------------------------------------------------}

interface

uses
  Classes, DsgnIntf;

 type
   TStringListItem = class(TCollectionItem)
   private
     { Private declarations }
     FStrings: TStrings;
     FTitle: string;
     FOnChanging: TNotifyEvent;
     FOnChange: TNotifyEvent;
     procedure SetStrings(const Value: TStrings);
     procedure SetTitle(const Value: string);
     procedure SetDuplicates(const Value: TDuplicates);
     procedure SetSorted(const Value: Boolean);
     function GetDuplicates: TDuplicates;
     function GetSorted: Boolean;
     procedure SetOnChange(const Value: TNotifyEvent);
     procedure SetOnChanging(const Value: TNotifyEvent);

     procedure DoOnChange(Sender : TObject);
     procedure DoOnChanging(Sender : TObject);
   protected
     { Protected declarations }
     function GetDisplayName: string; override;
   public
     { Public declarations }
     procedure Assign(Source : TPersistent); override;
     constructor Create(AOwner: TCollection); override;
     destructor Destroy; override;
   published
     { Properties of the Collection }
     property Strings : TStrings read FStrings write SetStrings;
     property Title : string read FTitle write SetTitle;
     property Sorted : Boolean read GetSorted write SetSorted;
     property Duplicates : TDuplicates read GetDuplicates write SetDuplicates;
     property OnChange : TNotifyEvent read FOnChange write SetOnChange;
     property OnChanging : TNotifyEvent read FOnChanging write SetOnChanging;
   end;

   TStringListItems = class(TCollection)
   protected
     { Protected declarations }
     FOwner : TComponent;
     function GetItem(Index: Integer): TStringListItem;
     procedure SetItem(Index: Integer; Value: TStringListItem);
     function GetOwner: TPersistent; override;
     { Design-time editor support }
     function GetAttrCount: Integer; override;
     function GetAttr(Index: Integer): string; override;
     function GetItemAttr(Index, ItemIndex: Integer): string; override;
   public
     function Add: TStringListItem;
     property Items[Index: Integer]: TStringListItem read GetItem write SetItem; default;
     Constructor Create(AOwner : TComponent); virtual;
   end;

   TStringListHolder = Class(TComponent)
   private
    FItems: TStringListItems;
    procedure SetItems(const Value: TStringListItems);
   public
     constructor Create(AOwner : TComponent); override;
     destructor Destroy; override;
   published
     property Items : TStringListItems read FItems write SetItems;
   end;

   TItemsEditor = class(TDefaultEditor)
   public
     procedure EditProperty(PropertyEditor: TpropertyEditor; var Continue,
       FreeEditor: Boolean); override;
     function GetVerbCount: Integer; override;
     function GetVerb(Index: Integer):String; override;
     procedure ExecuteVerb (Index: Integer); override;
   end;

procedure Register;

implementation

uses SysUtils;

{ TStringListItem }

procedure Register;
begin
   RegisterComponents('UKBUG', [TStringListHolder]);
   RegisterComponentEditor(TStringListHolder, TItemsEditor);
end;

procedure TStringListItem.Assign(Source: TPersistent);
begin
  if (Source is TStringListItem) then
  begin
    Self.Strings.Assign(TStringListItem(Source).Strings);
    Self.Title := TStringListItem(Source).Title;
  end
  else
    inherited Assign(Source);
end;

constructor TStringListItem.Create(AOwner: TCollection);
begin
  inherited Create(AOwner);
  FStrings := TStringList.Create;
  (FStrings as TStringList).OnChange := DoOnChange;
  (FStrings as TStringList).OnChanging := DoOnChanging;
  FTitle := 'StringList '+ IntToStr(Index);
end;

destructor TStringListItem.Destroy;
begin
  FStrings.Free;
  inherited;
end;


procedure TStringListItem.DoOnChange(Sender: TObject);
begin
  if Assigned(OnChange) then OnChange(Sender);
end;

procedure TStringListItem.DoOnChanging(Sender: TObject);
begin
  if Assigned(OnChanging) then OnChanging(Sender);
end;

function TStringListItem.GetDisplayName: string;
begin
  if FTitle <> '' then
    Result := FTitle
  else
    Result := 'StringList ' + IntToStr(Index);
end;

function TStringListItem.GetDuplicates: TDuplicates;
begin
  if Assigned(FStrings) then
    Result := (FStrings as TStringList).Duplicates
  else
    Result := dupAccept;
end;

function TStringListItem.GetSorted: Boolean;
begin
  if Assigned(FStrings) then
    Result := (FStrings as TStringList).Sorted
  else
    Result := False;
end;

procedure TStringListItem.SetDuplicates(const Value: TDuplicates);
begin
  if Assigned(FStrings) then
    (FStrings as TStringList).Duplicates := Value;
end;

procedure TStringListItem.SetOnChange(const Value: TNotifyEvent);
begin
  FOnChange := Value;
end;

procedure TStringListItem.SetOnChanging(const Value: TNotifyEvent);
begin
  FOnChanging := Value;
end;

procedure TStringListItem.SetSorted(const Value: Boolean);
begin
  if Assigned(FStrings) then
    (FStrings as TStringList).Sorted := Value;
end;

procedure TStringListItem.SetStrings(const Value: TStrings);
begin
  FStrings.Assign(Value);
end;

procedure TStringListItem.SetTitle(const Value: string);
begin
  FTitle := Value;
end;

{ TStringListItems }

constructor TStringListItems.Create(AOwner: TComponent);
begin
  FOwner := AOwner;
  inherited Create(TStringListItem);
end;

function TStringListItems.GetAttr(Index: Integer): string;
begin
  case Index of
    0: Result := 'Title';
    1: Result := 'Count';
    2: Result := 'Length';
    3: Result := 'Sorted';
  else Result := 'Unknown';
  end;
end;

function TStringListItems.GetAttrCount: Integer;
begin
  Result := 4;
end;

function TStringListItems.GetItem(Index: Integer): TStringListItem;
begin
  Result := TStringListItem(inherited GetItem(Index));
end;

procedure TStringListItems.SetItem(Index: Integer; Value: TStringListItem);
begin
  inherited SetItem(Index, Value);
end;

function TStringListItems.GetItemAttr(Index, ItemIndex: Integer): string;
var
  Item : TStringListItem;

  function BoolToStr(bln : Boolean) : string;
  begin
    if bln then
      Result := 'True'
    else
      Result := 'False';
  end;

begin
  Item := (Items[ItemIndex] as TStringListItem);
  case Index of
    0: Result := Item.Title;
    1: Result := IntToStr(Item.Strings.Count);
    2: Result := IntToStr(Length(Item.Strings.Text));
    3: Result := BoolToStr((Item.Strings as TStringList).Sorted);
  end;
end;

function TStringListItems.GetOwner: TPersistent;
begin
  Result := FOwner;
end;


function TStringListItems.Add: TStringListItem;
begin
  Result := TStringListItem(inherited Add);
end;

{ TStringListHolder }

constructor TStringListHolder.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FItems := TStringListItems.Create(Self);
end;

destructor TStringListHolder.Destroy;
begin
  FItems.Free;
  inherited;
end;

procedure TStringListHolder.SetItems(const Value: TStringListItems);
begin
  FItems.Assign(Value);
end;

{ TItemsEditor }

procedure TItemsEditor.EditProperty(PropertyEditor: TpropertyEditor;
  var Continue, FreeEditor: Boolean);
begin
  if (PropertyEditor.GetName = 'Items') then
  begin
    PropertyEditor.Edit;
    Continue := False;
  end;
end;

procedure TItemsEditor.ExecuteVerb(Index: Integer);
begin
  Edit;
end;

function TItemsEditor.GetVerb(Index: Integer): String;
begin
  Result := 'Edit Items...';
end;

function TItemsEditor.GetVerbCount: Integer;
begin
  Result := 1;
end;

end.
